@@ -5,6 +5,7 @@ Functions
 
 -   array                      - NumPy Array construction
 -   zeros                      - Return an array of all zeros
+-   empty                      - Return an unitialized array (200x faster than zeros)
 -   shape                      - Return shape of sequence or array
 -   rank                       - Return number of dimensions
 -   size                       - Return number of elements in entire array or a
@@ -58,7 +59,6 @@ More Functions:
 -   where                      - Construct array from binary result
 -   compress                   - Elements of array where condition is true
 -   clip                       - Clip array between two values
--   zeros                      - Array of all zeros
 -   ones                       - Array of all ones
 -   identity                   - 2-D identity array (matrix)
 
@@ -125,6 +125,7 @@ arrayrange = multiarray.arange
 
 array = multiarray.array
 zeros = multiarray.zeros
+empty = multiarray.empty
 
 def asarray(a, typecode=None, savespace=0):
     """asarray(a,typecode=None) returns a as a NumPy array.  Unlike array(),
@@ -205,6 +206,7 @@ def convolve(a,v,mode=2):
     return cross_correlate(a,asarray(v)[::-1],mode)
 
 ArrayType = multiarray.arraytype
+UfuncType = type(sin)
 
 def swapaxes(a, axis1, axis2):
     """swapaxes(a, axis1, axis2) returns array a with axis1 and axis2
@@ -292,7 +294,12 @@ def argmin(a, axis=-1):
     """argmin(a,axis=-1) returns the indices to the minimum value of the
     1-D arrays along the given axis.    
     """
-    a = -array(a, copy=0)
+    arra = array(a,copy=0)
+    type = arra.typecode()
+    num = array(0,type)
+    if type in ['bwu']:
+        num = -array(1,type)
+    a = num-arra
     n = len(a.shape)
     if axis < 0: axis += n
     if axis < 0 or axis >= n:
@@ -339,6 +346,18 @@ def dot(a, b):
         else:
             raise TypeError, detail or "invalid types for dot"
 
+def vdot(a, b):
+    """Returns the dot product of 2 vectors (or anything that can be made into
+       a vector). NB: this is not the same as `dot`, as it takes the conjugate
+       of its first argument if complex and always returns a scalar."""
+    return multiarray.matrixproduct(conjugate(ravel(a)),
+                                    ravel(b))
+
+# try to import blas optimized dot, innerproduct and vdot, if available
+try:
+    from dotblas import dot, innerproduct, vdot
+except ImportError: pass
+        
 #This is obsolete, don't use in new code
 matrixmultiply = dot
 
@@ -777,14 +796,3 @@ def average (a, axis=0, weights=None, returned = 0):
     else:
         return n/d
 
-def vdot(a, b):
-    """Returns the dot product of 2 vectors (or anything that can be made into
-       a vector). NB: this is not the same as `dot`, as it takes the conjugate
-       of its first argument if complex and always returns a scalar."""
-    return multiarray.matrixproduct(conjugate(ravel(a)),
-                                    ravel(b))
-
-# try to import blas optimized dot, innerproduct and vdot, if available
-try:
-    from dotblas import dot, innerproduct, vdot
-except ImportError: pass
