@@ -93,7 +93,7 @@ arr_histogram(PyObject *self, PyObject *args)
     double * weights , * dans ;
 
     Py_Try(PyArg_ParseTuple(args, "O|O", &list, &weight));
-    GET_ARR(lst,list,PyArray_LONG,1);
+    GET_ARR(lst,list,PyArray_INT,1);
     len = A_SIZE (lst) ;
     numbers = (int *) A_DATA (lst) ;
     mxi = mxx (numbers, len) ;
@@ -106,7 +106,7 @@ arr_histogram(PyObject *self, PyObject *args)
     if (weight == NULL)
 	{
 	    Py_Try(ans =
-		   (PyArrayObject *) PyArray_FromDims (1, &ans_size, PyArray_LONG));
+		   (PyArrayObject *) PyArray_FromDims (1, &ans_size, PyArray_INT));
 	    ians = (int *) A_DATA (ans) ;
 	    for (i = 0 ; i < len ; i++)
 		ians [numbers [i]] += 1 ;
@@ -200,7 +200,7 @@ arr_array_set(PyObject *self, PyObject *args)
 	SETERR ("array_set: target must have one or two dimensions.");
 	return NULL ;
     }
-    GET_ARR(subsarr,subsarg,PyArray_LONG,1);
+    GET_ARR(subsarr,subsarg,PyArray_INT,1);
     isubs = (int *)A_DATA(subsarr);
     len = A_SIZE(subsarr);
     mn = mnx (isubs, len);
@@ -515,7 +515,7 @@ arr_index_sort(PyObject *self, PyObject *args)
     Py_Try(PyArg_ParseTuple(args, "O", &list));
     GET_ARR(alist,list,PyArray_DOUBLE,1);
     len = A_SIZE(alist);
-    Py_Try(ilist = (PyArrayObject *) PyArray_FromDims (1, &len, PyArray_LONG));
+    Py_Try(ilist = (PyArrayObject *) PyArray_FromDims (1, &len, PyArray_INT));
     isubs = (int *) A_DATA (ilist);
     for ( i = 0 ; i < len ; i ++ )
 	isubs [i] = i ;
@@ -801,7 +801,7 @@ arr_zmin_zmax(PyObject *self, PyObject *args)
     Py_Try(PyArg_ParseTuple(args, "OO", &zobj, &iregobj));
     GET_ARR (zarr, zobj, PyArray_DOUBLE, 2);
     if (! (iregarr = (PyArrayObject *)PyArray_ContiguousFromObject(iregobj,
-								   PyArray_LONG, 2, 2))) {
+								   PyArray_INT, 2, 2))) {
 	Py_DECREF (zarr);
 	return NULL;
     }
@@ -1115,8 +1115,8 @@ static PyObject * arr_find_mask (PyObject * self, PyObject * args)
     int i, j, k, l, ll, ifs, imask, ntotal, ne, nv, ans_size ;
 
     Py_Try (PyArg_ParseTuple ( args, "OO", & fso, & node_edgeso ) ) ;
-    GET_ARR (fsa, fso, PyArray_LONG, 2) ;
-    GET_ARR (node_edgesa, node_edgeso, PyArray_LONG, 2) ;
+    GET_ARR (fsa, fso, PyArray_INT, 2) ;
+    GET_ARR (node_edgesa, node_edgeso, PyArray_INT, 2) ;
     fs = (int *) A_DATA (fsa) ;
     node_edges = (int *) A_DATA (node_edgesa) ;
     ntotal = A_DIM (fsa, 0) ;
@@ -1130,7 +1130,7 @@ static PyObject * arr_find_mask (PyObject * self, PyObject * args)
     ne = A_DIM (node_edgesa, 1) ;
     ans_size = ntotal * ne ;
     Py_Try (maska = (PyArrayObject *) PyArray_FromDims
-	    (1, & ans_size, PyArray_LONG)) ;
+	    (1, & ans_size, PyArray_INT)) ;
     mask = (int *) A_DATA (maska) ;
 
     for (i = 0, ifs = 0, imask = 0 ; i < ntotal ;
@@ -1322,7 +1322,7 @@ arr_construct3 (PyObject * self, PyObject * args)
 
     /*    dbg = fopen ("dbg","w"); */
     Py_Try (PyArg_ParseTuple ( args, "Oi", & masko, & itype ) ) ;
-    GET_ARR (maska, masko, PyArray_LONG, 1) ;
+    GET_ARR (maska, masko, PyArray_INT, 1) ;
     mask = (int *) A_DATA (maska) ;
     permute_dims [0] = ne = no_edges [itype] ;
     permute_dims [1] = pt = powers [itype] ;
@@ -1333,7 +1333,7 @@ arr_construct3 (PyObject * self, PyObject * args)
 	return NULL ;
     }
     Py_Try(permutea =
-	   (PyArrayObject *) PyArray_FromDims (2, permute_dims, PyArray_LONG));
+	   (PyArrayObject *) PyArray_FromDims (2, permute_dims, PyArray_INT));
     permute = (int *) A_DATA (permutea) ;
     for ( i = 0 ; i < pt ; i ++, permute ++, mask += ne )
 	{
@@ -1361,7 +1361,7 @@ arr_to_corners (PyObject * self, PyObject * args)
     Py_Try (PyArg_ParseTuple (args, "OOi", & oarr, & onv, & sum_nv));
     GET_ARR (aarr, oarr, PyArray_DOUBLE, 1) ;
     if isARRAY (onv) {
-	GET_ARR (anv, onv, PyArray_LONG, 1) ;
+	GET_ARR (anv, onv, PyArray_INT, 1) ;
     }
     else {
 	ERRSS ("The second argument must be an Int array") ;
