@@ -13,7 +13,7 @@
   performance for heterogeneity.
 */
 
-/* $Id: arrayobject.c,v 1.72 2004/11/30 18:49:18 teoliphant Exp $ */
+/* $Id: arrayobject.c,v 1.73 2005/02/08 22:59:18 teoliphant Exp $ */
 
 #include "Python.h"
 #include <stddef.h>
@@ -2584,6 +2584,7 @@ PyObject *PyArray_CopyFromObject(PyObject *op, int type, int min_depth, int max_
 }
 
 extern int PyArray_CanCastSafely(int fromtype, int totype) {
+
     if (fromtype == totype) return 1;
     if (totype == PyArray_OBJECT) return 1;
     if (fromtype == PyArray_OBJECT) return 0;
@@ -2603,8 +2604,12 @@ extern int PyArray_CanCastSafely(int fromtype, int totype) {
     case PyArray_UINT:
         return (totype > PyArray_FLOAT);
     case PyArray_LONG:
-	/*Shouldn't allow Longs to be cast to Ints, not safe on 64-bit machines!*/
-	return (totype > PyArray_LONG) && (totype != PyArray_FLOAT) && (totype != PyArray_CFLOAT); 
+	/* Hopefully this will allow LONGs to be cast to INTs on 32 bit platforms where 
+	   they can be, but not on 64-bit platforms where it is not safe */
+	if (sizeof(long) == sizeof(int)) 
+	    return (totype == PyArray_INT) || (totype == PyArray_DOUBLE) || (totype == PyArray_CDOUBLE);
+	else
+	    return (totype == PyArray_DOUBLE) || (totype == PyArray_CDOUBLE);
     case PyArray_FLOAT:
 	return (totype > fromtype);
     case PyArray_DOUBLE:
@@ -2683,7 +2688,7 @@ extern PyObject * PyArray_Reshape(PyArrayObject *self, PyObject *shape) {
 	return NULL;
     }
 	
-    if (PyArray_As1D(&shape, (char **)&dimensions, &n, PyArray_LONG) == -1) return NULL;
+    if (PyArray_As1D(&shape, (char **)&dimensions, &n, PyArray_INT) == -1) return NULL;
 	
     s_known = 1;
     i_unknown = -1;
@@ -2734,7 +2739,8 @@ extern PyObject * PyArray_Reshape(PyArrayObject *self, PyObject *shape) {
 extern PyObject *PyArray_Take(PyObject *self0, PyObject *indices0, int axis) {
     PyArrayObject *self, *indices, *ret;
     int nd, shape[MAX_DIMS];
-    int i, j, chunk, n, m, max_item, tmp;
+    int i, j, chunk, n, m, max_item;
+    long tmp;
     char *src, *dest;
 	
     indices = ret = NULL;
@@ -2807,7 +2813,8 @@ extern PyObject *PyArray_Take(PyObject *self0, PyObject *indices0, int axis) {
 extern PyObject *PyArray_Put(PyObject *self0, PyObject *indices0, 
                              PyObject* values0) {
     PyArrayObject  *indices, *values, *self;
-    int i, chunk, ni, max_item, nv, tmp; 
+    int i, chunk, ni, max_item, nv;
+    long tmp;
     char *src, *dest;
 
     indices = NULL;
@@ -2861,7 +2868,8 @@ extern PyObject *PyArray_Put(PyObject *self0, PyObject *indices0,
 extern PyObject *PyArray_PutMask(PyObject *self0, PyObject *mask0, 
                                  PyObject* values0) {
     PyArrayObject  *mask, *values, *self;
-    int i, chunk, ni, max_item, nv, tmp; 
+    int i, chunk, ni, max_item, nv;
+    long tmp;
     char *src, *dest;
 
     mask = NULL;
