@@ -2,6 +2,17 @@
 #include "Python.h"
 #include "Numeric/arrayobject.h"
 
+#ifndef CHAR_BIT
+#define CHAR_BIT 8
+#endif
+
+#ifndef INT_BIT
+#define INT_BIT (CHAR_BIT * sizeof(int))
+#endif
+
+#ifndef INT_MAX
+#define INT_MAX ((1 << (INT_BIT-2)) + ((1 << (INT_BIT-2))-1))
+#endif
 
 static void CHAR_to_CHAR(char *ip, int ipstep, char *op, int opstep, int n)
 {int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (char)*ip;}}
@@ -15,9 +26,15 @@ static void CHAR_to_SBYTE(char *ip, int ipstep, signed char *op, int opstep, int
 static void CHAR_to_SHORT(char *ip, int ipstep, short *op, int opstep, int n)
 {int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (short)*ip;}}
 
+static void CHAR_to_USHORT(char *ip, int ipstep, unsigned short *op, int opstep, int n)
+{int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (unsigned short)*ip;}}
+
 static void CHAR_to_INT(char *ip, int ipstep, int *op, int opstep, int n)
 {int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (int)*ip;}}
 
+static void CHAR_to_UINT(char *ip, int ipstep, unsigned int *op, int opstep, int n)
+{int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (unsigned int)*ip;}}
+
 static void CHAR_to_LONG(char *ip, int ipstep, long *op, int opstep, int n)
 {int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (long)*ip;}}
 
@@ -46,7 +63,9 @@ static PyArray_Descr CHAR_Descr = { {
     (PyArray_VectorUnaryFunc*)CHAR_to_UBYTE, 
     (PyArray_VectorUnaryFunc*)CHAR_to_SBYTE, 
     (PyArray_VectorUnaryFunc*)CHAR_to_SHORT, 
+    (PyArray_VectorUnaryFunc*)CHAR_to_USHORT, 
     (PyArray_VectorUnaryFunc*)CHAR_to_INT, 
+    (PyArray_VectorUnaryFunc*)CHAR_to_UINT, 
     (PyArray_VectorUnaryFunc*)CHAR_to_LONG, 
     (PyArray_VectorUnaryFunc*)CHAR_to_FLOAT, 
     (PyArray_VectorUnaryFunc*)CHAR_to_DOUBLE, 
@@ -70,9 +89,15 @@ static void UBYTE_to_SBYTE(unsigned char *ip, int ipstep, signed char *op, int o
 static void UBYTE_to_SHORT(unsigned char *ip, int ipstep, short *op, int opstep, int n)
 {int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (short)*ip;}}
 
+static void UBYTE_to_USHORT(unsigned char *ip, int ipstep, unsigned short *op, int opstep, int n)
+{int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (unsigned short)*ip;}}
+
 static void UBYTE_to_INT(unsigned char *ip, int ipstep, int *op, int opstep, int n)
 {int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (int)*ip;}}
 
+static void UBYTE_to_UINT(unsigned char *ip, int ipstep, unsigned int *op, int opstep, int n)
+{int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (unsigned int)*ip;}}
+
 static void UBYTE_to_LONG(unsigned char *ip, int ipstep, long *op, int opstep, int n)
 {int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (long)*ip;}}
 
@@ -101,7 +126,9 @@ static PyArray_Descr UBYTE_Descr = { {
     (PyArray_VectorUnaryFunc*)UBYTE_to_UBYTE, 
     (PyArray_VectorUnaryFunc*)UBYTE_to_SBYTE, 
     (PyArray_VectorUnaryFunc*)UBYTE_to_SHORT, 
+    (PyArray_VectorUnaryFunc*)UBYTE_to_USHORT, 
     (PyArray_VectorUnaryFunc*)UBYTE_to_INT, 
+    (PyArray_VectorUnaryFunc*)UBYTE_to_UINT, 
     (PyArray_VectorUnaryFunc*)UBYTE_to_LONG, 
     (PyArray_VectorUnaryFunc*)UBYTE_to_FLOAT, 
     (PyArray_VectorUnaryFunc*)UBYTE_to_DOUBLE, 
@@ -125,9 +152,15 @@ static void SBYTE_to_SBYTE(signed char *ip, int ipstep, signed char *op, int ops
 static void SBYTE_to_SHORT(signed char *ip, int ipstep, short *op, int opstep, int n)
 {int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (short)*ip;}}
 
+static void SBYTE_to_USHORT(signed char *ip, int ipstep, unsigned short *op, int opstep, int n)
+{int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (unsigned short)*ip;}}
+
 static void SBYTE_to_INT(signed char *ip, int ipstep, int *op, int opstep, int n)
 {int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (int)*ip;}}
 
+static void SBYTE_to_UINT(signed char *ip, int ipstep, unsigned int *op, int opstep, int n)
+{int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (unsigned int)*ip;}}
+
 static void SBYTE_to_LONG(signed char *ip, int ipstep, long *op, int opstep, int n)
 {int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (long)*ip;}}
 
@@ -156,7 +189,9 @@ static PyArray_Descr SBYTE_Descr = { {
     (PyArray_VectorUnaryFunc*)SBYTE_to_UBYTE, 
     (PyArray_VectorUnaryFunc*)SBYTE_to_SBYTE, 
     (PyArray_VectorUnaryFunc*)SBYTE_to_SHORT, 
+    (PyArray_VectorUnaryFunc*)SBYTE_to_USHORT, 
     (PyArray_VectorUnaryFunc*)SBYTE_to_INT, 
+    (PyArray_VectorUnaryFunc*)SBYTE_to_UINT, 
     (PyArray_VectorUnaryFunc*)SBYTE_to_LONG, 
     (PyArray_VectorUnaryFunc*)SBYTE_to_FLOAT, 
     (PyArray_VectorUnaryFunc*)SBYTE_to_DOUBLE, 
@@ -168,6 +203,69 @@ static PyArray_Descr SBYTE_Descr = { {
 				     PyArray_SBYTE, sizeof(signed char), NULL, NULL, '1'};
 
 
+static void USHORT_to_CHAR(unsigned short *ip, int ipstep, char *op, int opstep, int n)
+{int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (char)*ip;}}
+
+static void USHORT_to_UBYTE(unsigned short *ip, int ipstep, unsigned char *op, int opstep, int n)
+{int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (unsigned char)*ip;}}
+
+static void USHORT_to_SBYTE(unsigned short *ip, int ipstep, signed char *op, int opstep, int n)
+{int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (signed char)*ip;}}
+
+static void USHORT_to_SHORT(unsigned short *ip, int ipstep, short *op, int opstep, int n)
+{int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (short)*ip;}}
+
+static void USHORT_to_USHORT(unsigned short *ip, int ipstep, unsigned short *op, int opstep, int n)
+{int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (unsigned short)*ip;}}
+
+static void USHORT_to_INT(unsigned short *ip, int ipstep, int *op, int opstep, int n)
+{int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (int)*ip;}}
+
+static void USHORT_to_UINT(unsigned short *ip, int ipstep, unsigned int *op, int opstep, int n)
+{int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (unsigned int)*ip;}}
+
+static void USHORT_to_LONG(unsigned short *ip, int ipstep, long *op, int opstep, int n)
+{int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (long)*ip;}}
+
+static void USHORT_to_FLOAT(unsigned short *ip, int ipstep, float *op, int opstep, int n)
+{int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (float)*ip;}}
+
+static void USHORT_to_DOUBLE(unsigned short *ip, int ipstep, double *op, int opstep, int n)
+{int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (double)*ip;}}
+
+static void USHORT_to_CFLOAT(unsigned short *ip, int ipstep, float *op, int opstep, int n)
+{int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep*2) {op[0] = (float)*ip; op[1]=0.0;}}
+
+static void USHORT_to_CDOUBLE(unsigned short *ip, int ipstep, double *op, int opstep, int n)
+{int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep*2) {op[0] = (double)*ip; op[1]=0.0;}}
+
+static void USHORT_to_OBJECT(unsigned short *ip, int ipstep, PyObject  **op, int opstep, int n)
+{int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {op[0] = PyInt_FromLong((long)*((unsigned short *)ip));}}
+
+static PyObject * USHORT_getitem(char *ip) {return PyInt_FromLong((long)*((unsigned short *)ip));}
+
+static int USHORT_setitem(PyObject *op, char *ov) {*((unsigned short *)ov)=(unsigned short)PyInt_AsLong(op);return PyErr_Occurred() ? -1:0;}
+
+static PyArray_Descr USHORT_Descr = { {
+
+	(PyArray_VectorUnaryFunc*)USHORT_to_CHAR, 
+	(PyArray_VectorUnaryFunc*)USHORT_to_UBYTE, 
+	(PyArray_VectorUnaryFunc*)USHORT_to_SBYTE, 
+	(PyArray_VectorUnaryFunc*)USHORT_to_SHORT, 
+	(PyArray_VectorUnaryFunc*)USHORT_to_USHORT, 
+	(PyArray_VectorUnaryFunc*)USHORT_to_INT, 
+	(PyArray_VectorUnaryFunc*)USHORT_to_UINT, 
+	(PyArray_VectorUnaryFunc*)USHORT_to_LONG, 
+	(PyArray_VectorUnaryFunc*)USHORT_to_FLOAT, 
+	(PyArray_VectorUnaryFunc*)USHORT_to_DOUBLE, 
+	(PyArray_VectorUnaryFunc*)USHORT_to_CFLOAT, 
+	(PyArray_VectorUnaryFunc*)USHORT_to_CDOUBLE, 
+	(PyArray_VectorUnaryFunc*)USHORT_to_OBJECT, 
+},
+	USHORT_getitem, USHORT_setitem,
+	PyArray_USHORT, sizeof(unsigned short), NULL, NULL, 'w'};
+
+
 static void SHORT_to_CHAR(short *ip, int ipstep, char *op, int opstep, int n)
 {int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (char)*ip;}}
 
@@ -180,9 +278,15 @@ static void SHORT_to_SBYTE(short *ip, int ipstep, signed char *op, int opstep, i
 static void SHORT_to_SHORT(short *ip, int ipstep, short *op, int opstep, int n)
 {int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (short)*ip;}}
 
+static void SHORT_to_USHORT(short *ip, int ipstep, unsigned short *op, int opstep, int n)
+{int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (unsigned short)*ip;}}
+
 static void SHORT_to_INT(short *ip, int ipstep, int *op, int opstep, int n)
 {int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (int)*ip;}}
 
+static void SHORT_to_UINT(short *ip, int ipstep, unsigned int *op, int opstep, int n)
+{int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (unsigned int)*ip;}}
+
 static void SHORT_to_LONG(short *ip, int ipstep, long *op, int opstep, int n)
 {int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (long)*ip;}}
 
@@ -211,7 +315,9 @@ static PyArray_Descr SHORT_Descr = { {
     (PyArray_VectorUnaryFunc*)SHORT_to_UBYTE, 
     (PyArray_VectorUnaryFunc*)SHORT_to_SBYTE, 
     (PyArray_VectorUnaryFunc*)SHORT_to_SHORT, 
+    (PyArray_VectorUnaryFunc*)SHORT_to_USHORT, 
     (PyArray_VectorUnaryFunc*)SHORT_to_INT, 
+    (PyArray_VectorUnaryFunc*)SHORT_to_UINT, 
     (PyArray_VectorUnaryFunc*)SHORT_to_LONG, 
     (PyArray_VectorUnaryFunc*)SHORT_to_FLOAT, 
     (PyArray_VectorUnaryFunc*)SHORT_to_DOUBLE, 
@@ -235,9 +341,15 @@ static void INT_to_SBYTE(int *ip, int ipstep, signed char *op, int opstep, int n
 static void INT_to_SHORT(int *ip, int ipstep, short *op, int opstep, int n)
 {int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (short)*ip;}}
 
+static void INT_to_USHORT(int *ip, int ipstep, unsigned short *op, int opstep, int n)
+{int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (unsigned short)*ip;}}
+
 static void INT_to_INT(int *ip, int ipstep, int *op, int opstep, int n)
 {int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (int)*ip;}}
 
+static void INT_to_UINT(int *ip, int ipstep, unsigned int *op, int opstep, int n)
+{int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (unsigned int)*ip;}}
+
 static void INT_to_LONG(int *ip, int ipstep, long *op, int opstep, int n)
 {int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (long)*ip;}}
 
@@ -266,7 +378,9 @@ static PyArray_Descr INT_Descr = { {
     (PyArray_VectorUnaryFunc*)INT_to_UBYTE, 
     (PyArray_VectorUnaryFunc*)INT_to_SBYTE, 
     (PyArray_VectorUnaryFunc*)INT_to_SHORT, 
+    (PyArray_VectorUnaryFunc*)INT_to_USHORT, 
     (PyArray_VectorUnaryFunc*)INT_to_INT, 
+    (PyArray_VectorUnaryFunc*)INT_to_UINT, 
     (PyArray_VectorUnaryFunc*)INT_to_LONG, 
     (PyArray_VectorUnaryFunc*)INT_to_FLOAT, 
     (PyArray_VectorUnaryFunc*)INT_to_DOUBLE, 
@@ -277,6 +391,81 @@ static PyArray_Descr INT_Descr = { {
 				   INT_getitem, INT_setitem,
 				   PyArray_INT, sizeof(int), NULL, NULL, 'i'};
 
+static void UINT_to_CHAR(unsigned int *ip, int ipstep, char *op, int opstep, int n)
+{int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (char)*ip;}}
+
+static void UINT_to_UBYTE(unsigned int *ip, int ipstep, unsigned char *op, int opstep, int n)
+{int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (unsigned char)*ip;}}
+
+static void UINT_to_SBYTE(unsigned int *ip, int ipstep, signed char *op, int opstep, int n)
+{int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (signed char)*ip;}}
+
+static void UINT_to_SHORT(unsigned int *ip, int ipstep, short *op, int opstep, int n)
+{int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (short)*ip;}}
+
+static void UINT_to_USHORT(unsigned int *ip, int ipstep, unsigned short *op, int opstep, int n)
+{int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (unsigned short)*ip;}}
+
+static void UINT_to_INT(unsigned int *ip, int ipstep, int *op, int opstep, int n)
+{int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (int)*ip;}}
+
+static void UINT_to_UINT(unsigned int *ip, int ipstep, unsigned int *op, int opstep, int n)
+{int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (unsigned int)*ip;}}
+
+static void UINT_to_LONG(unsigned int *ip, int ipstep, long *op, int opstep, int n)
+{int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (long)*ip;}}
+
+static void UINT_to_FLOAT(unsigned int *ip, int ipstep, float *op, int opstep, int n)
+{int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (float)*ip;}}
+
+static void UINT_to_DOUBLE(unsigned int *ip, int ipstep, double *op, int opstep, int n)
+{int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (double)*ip;}}
+
+static void UINT_to_CFLOAT(unsigned int *ip, int ipstep, float *op, int opstep, int n)
+{int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep*2) {op[0] = (float)*ip; op[1]=0.0;}}
+
+static void UINT_to_CDOUBLE(unsigned int *ip, int ipstep, double *op, int opstep, int n)
+{int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep*2) {op[0] = (double)*ip; op[1]=0.0;}}
+
+static void UINT_to_OBJECT(unsigned int *ip, int ipstep, PyObject  **op, int opstep, int n)
+{int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {op[0] = PyInt_FromLong((long)*ip);}}
+
+static PyObject * UINT_getitem(char *ip) {
+	if (*((unsigned int *)ip) > (unsigned int) INT_MAX) {
+		return PyLong_FromUnsignedLong((unsigned long)*((unsigned int *)ip));
+	} else {
+		return PyInt_FromLong((long)*((unsigned int *)ip));
+	}
+}
+
+static int UINT_setitem(PyObject *op, char *ov) {
+	if (PyLong_Check(op)) {
+		*((unsigned int *)ov)=(unsigned int)PyLong_AsUnsignedLong(op);
+	} else {
+		*((unsigned int *)ov)=(unsigned int)PyInt_AsLong(op);
+	}
+	return PyErr_Occurred() ? -1:0;
+}
+
+static PyArray_Descr UINT_Descr = { {
+
+    (PyArray_VectorUnaryFunc*)UINT_to_CHAR, 
+    (PyArray_VectorUnaryFunc*)UINT_to_UBYTE, 
+    (PyArray_VectorUnaryFunc*)UINT_to_SBYTE, 
+    (PyArray_VectorUnaryFunc*)UINT_to_SHORT, 
+    (PyArray_VectorUnaryFunc*)UINT_to_USHORT, 
+    (PyArray_VectorUnaryFunc*)UINT_to_INT, 
+    (PyArray_VectorUnaryFunc*)UINT_to_UINT, 
+    (PyArray_VectorUnaryFunc*)UINT_to_LONG, 
+    (PyArray_VectorUnaryFunc*)UINT_to_FLOAT, 
+    (PyArray_VectorUnaryFunc*)UINT_to_DOUBLE, 
+    (PyArray_VectorUnaryFunc*)UINT_to_CFLOAT, 
+    (PyArray_VectorUnaryFunc*)UINT_to_CDOUBLE, 
+    (PyArray_VectorUnaryFunc*)UINT_to_OBJECT, 
+},
+				   UINT_getitem, UINT_setitem,
+				   PyArray_UINT, sizeof(unsigned int), NULL, NULL, 'u'};
+
 
 static void LONG_to_CHAR(long *ip, int ipstep, char *op, int opstep, int n)
 {int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (char)*ip;}}
@@ -290,9 +479,15 @@ static void LONG_to_SBYTE(long *ip, int ipstep, signed char *op, int opstep, int
 static void LONG_to_SHORT(long *ip, int ipstep, short *op, int opstep, int n)
 {int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (short)*ip;}}
 
+static void LONG_to_USHORT(long *ip, int ipstep, unsigned short *op, int opstep, int n)
+{int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (unsigned short)*ip;}}
+
 static void LONG_to_INT(long *ip, int ipstep, int *op, int opstep, int n)
 {int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (int)*ip;}}
 
+static void LONG_to_UINT(long *ip, int ipstep, unsigned int *op, int opstep, int n)
+{int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (unsigned int)*ip;}}
+
 static void LONG_to_LONG(long *ip, int ipstep, long *op, int opstep, int n)
 {int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (long)*ip;}}
 
@@ -321,7 +516,9 @@ static PyArray_Descr LONG_Descr = { {
     (PyArray_VectorUnaryFunc*)LONG_to_UBYTE, 
     (PyArray_VectorUnaryFunc*)LONG_to_SBYTE, 
     (PyArray_VectorUnaryFunc*)LONG_to_SHORT, 
+    (PyArray_VectorUnaryFunc*)LONG_to_USHORT, 
     (PyArray_VectorUnaryFunc*)LONG_to_INT, 
+    (PyArray_VectorUnaryFunc*)LONG_to_UINT, 
     (PyArray_VectorUnaryFunc*)LONG_to_LONG, 
     (PyArray_VectorUnaryFunc*)LONG_to_FLOAT, 
     (PyArray_VectorUnaryFunc*)LONG_to_DOUBLE, 
@@ -345,9 +542,15 @@ static void FLOAT_to_SBYTE(float *ip, int ipstep, signed char *op, int opstep, i
 static void FLOAT_to_SHORT(float *ip, int ipstep, short *op, int opstep, int n)
 {int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (short)*ip;}}
 
+static void FLOAT_to_USHORT(float *ip, int ipstep, unsigned short *op, int opstep, int n)
+{int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (unsigned short)*ip;}}
+
 static void FLOAT_to_INT(float *ip, int ipstep, int *op, int opstep, int n)
 {int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (int)*ip;}}
 
+static void FLOAT_to_UINT(float *ip, int ipstep, unsigned int *op, int opstep, int n)
+{int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (unsigned int)*ip;}}
+
 static void FLOAT_to_LONG(float *ip, int ipstep, long *op, int opstep, int n)
 {int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (long)*ip;}}
 
@@ -376,7 +579,9 @@ static PyArray_Descr FLOAT_Descr = { {
     (PyArray_VectorUnaryFunc*)FLOAT_to_UBYTE, 
     (PyArray_VectorUnaryFunc*)FLOAT_to_SBYTE, 
     (PyArray_VectorUnaryFunc*)FLOAT_to_SHORT, 
+    (PyArray_VectorUnaryFunc*)FLOAT_to_USHORT, 
     (PyArray_VectorUnaryFunc*)FLOAT_to_INT, 
+    (PyArray_VectorUnaryFunc*)FLOAT_to_UINT, 
     (PyArray_VectorUnaryFunc*)FLOAT_to_LONG, 
     (PyArray_VectorUnaryFunc*)FLOAT_to_FLOAT, 
     (PyArray_VectorUnaryFunc*)FLOAT_to_DOUBLE, 
@@ -400,9 +605,15 @@ static void DOUBLE_to_SBYTE(double *ip, int ipstep, signed char *op, int opstep,
 static void DOUBLE_to_SHORT(double *ip, int ipstep, short *op, int opstep, int n)
 {int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (short)*ip;}}
 
+static void DOUBLE_to_USHORT(double *ip, int ipstep, unsigned short *op, int opstep, int n)
+{int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (unsigned short)*ip;}}
+
 static void DOUBLE_to_INT(double *ip, int ipstep, int *op, int opstep, int n)
 {int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (int)*ip;}}
 
+static void DOUBLE_to_UINT(double *ip, int ipstep, unsigned int *op, int opstep, int n)
+{int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (unsigned int)*ip;}}
+
 static void DOUBLE_to_LONG(double *ip, int ipstep, long *op, int opstep, int n)
 {int i; for(i=0;i<n;i++,ip+=ipstep,op+=opstep) {*op = (long)*ip;}}
 
@@ -431,7 +642,9 @@ static PyArray_Descr DOUBLE_Descr = { {
     (PyArray_VectorUnaryFunc*)DOUBLE_to_UBYTE, 
     (PyArray_VectorUnaryFunc*)DOUBLE_to_SBYTE, 
     (PyArray_VectorUnaryFunc*)DOUBLE_to_SHORT, 
+    (PyArray_VectorUnaryFunc*)DOUBLE_to_USHORT, 
     (PyArray_VectorUnaryFunc*)DOUBLE_to_INT, 
+    (PyArray_VectorUnaryFunc*)DOUBLE_to_UINT, 
     (PyArray_VectorUnaryFunc*)DOUBLE_to_LONG, 
     (PyArray_VectorUnaryFunc*)DOUBLE_to_FLOAT, 
     (PyArray_VectorUnaryFunc*)DOUBLE_to_DOUBLE, 
@@ -455,9 +668,15 @@ static void CFLOAT_to_SBYTE(float *ip, int ipstep, signed char *op, int opstep,
 static void CFLOAT_to_SHORT(float *ip, int ipstep, short *op, int opstep, int n)
 {int i; for(i=0;i<n;i++,ip+=ipstep*2,op+=opstep) {*op = (short)*ip; }}
 
+static void CFLOAT_to_USHORT(float *ip, int ipstep, unsigned short *op, int opstep, int n)
+{int i; for(i=0;i<n;i++,ip+=ipstep*2,op+=opstep) {*op = (unsigned short)*ip; }}
+
 static void CFLOAT_to_INT(float *ip, int ipstep, int *op, int opstep, int n)
 {int i; for(i=0;i<n;i++,ip+=ipstep*2,op+=opstep) {*op = (int)*ip; }}
 
+static void CFLOAT_to_UINT(float *ip, int ipstep, unsigned int *op, int opstep, int n)
+{int i; for(i=0;i<n;i++,ip+=ipstep*2,op+=opstep) {*op = (unsigned int)*ip; }}
+
 static void CFLOAT_to_LONG(float *ip, int ipstep, long *op, int opstep, int n)
 {int i; for(i=0;i<n;i++,ip+=ipstep*2,op+=opstep) {*op = (long)*ip; }}
 
@@ -481,7 +700,11 @@ static PyObject * CFLOAT_getitem(char *ip) {return PyComplex_FromDoubles((double
 static int CFLOAT_setitem(PyObject *op, char *ov) 
 {
     Py_complex oop;
-    oop = PyComplex_AsCComplex (op);
+    PyObject *op2;
+    if PyScalarArray_Check(op) op2 = ((PyArrayObject *)op)->descr->getitem(((PyArrayObject *)op)->data);
+    else { op2 = op; Py_INCREF(op);}
+    oop = PyComplex_AsCComplex (op2);
+    Py_DECREF(op2);
     if (PyErr_Occurred()) return -1;
     ((float *)ov)[0]= (float) oop.real;
     ((float *)ov)[1]= (float) oop.imag;
@@ -494,7 +717,9 @@ static PyArray_Descr CFLOAT_Descr = { {
     (PyArray_VectorUnaryFunc*)CFLOAT_to_UBYTE, 
     (PyArray_VectorUnaryFunc*)CFLOAT_to_SBYTE, 
     (PyArray_VectorUnaryFunc*)CFLOAT_to_SHORT, 
+    (PyArray_VectorUnaryFunc*)CFLOAT_to_USHORT, 
     (PyArray_VectorUnaryFunc*)CFLOAT_to_INT, 
+    (PyArray_VectorUnaryFunc*)CFLOAT_to_UINT, 
     (PyArray_VectorUnaryFunc*)CFLOAT_to_LONG, 
     (PyArray_VectorUnaryFunc*)CFLOAT_to_FLOAT, 
     (PyArray_VectorUnaryFunc*)CFLOAT_to_DOUBLE, 
@@ -518,9 +743,15 @@ static void CDOUBLE_to_SBYTE(double *ip, int ipstep, signed char *op, int opstep
 static void CDOUBLE_to_SHORT(double *ip, int ipstep, short *op, int opstep, int n)
 {int i; for(i=0;i<n;i++,ip+=ipstep*2,op+=opstep) {*op = (short)*ip; }}
 
+static void CDOUBLE_to_USHORT(double *ip, int ipstep, unsigned short *op, int opstep, int n)
+     {int i; for(i=0;i<n;i++,ip+=ipstep*2,op+=opstep) {*op = (unsigned short)*ip; }}
+
 static void CDOUBLE_to_INT(double *ip, int ipstep, int *op, int opstep, int n)
 {int i; for(i=0;i<n;i++,ip+=ipstep*2,op+=opstep) {*op = (int)*ip; }}
 
+static void CDOUBLE_to_UINT(double *ip, int ipstep, unsigned int *op, int opstep, int n)
+{int i; for(i=0;i<n;i++,ip+=ipstep*2,op+=opstep) {*op = (unsigned int)*ip; }}
+
 static void CDOUBLE_to_LONG(double *ip, int ipstep, long *op, int opstep, int n)
 {int i; for(i=0;i<n;i++,ip+=ipstep*2,op+=opstep) {*op = (long)*ip; }}
 
@@ -543,7 +774,11 @@ static PyObject * CDOUBLE_getitem(char *ip) {return PyComplex_FromDoubles((doubl
 
 static int CDOUBLE_setitem(PyObject *op, char *ov) {
     Py_complex oop;
-    oop = PyComplex_AsCComplex (op);
+    PyObject *op2;
+    if PyScalarArray_Check(op) op2 = ((PyArrayObject *)op)->descr->getitem(((PyArrayObject *)op)->data);
+    else { op2 = op; Py_INCREF(op);}
+    oop = PyComplex_AsCComplex (op2);
+    Py_DECREF(op2);
     if (PyErr_Occurred()) return -1;
     ((double *)ov)[0]= (double) oop.real;
     ((double *)ov)[1]= (double) oop.imag;
@@ -556,7 +791,9 @@ static PyArray_Descr CDOUBLE_Descr = { {
     (PyArray_VectorUnaryFunc*)CDOUBLE_to_UBYTE, 
     (PyArray_VectorUnaryFunc*)CDOUBLE_to_SBYTE, 
     (PyArray_VectorUnaryFunc*)CDOUBLE_to_SHORT, 
+    (PyArray_VectorUnaryFunc*)CDOUBLE_to_USHORT, 
     (PyArray_VectorUnaryFunc*)CDOUBLE_to_INT, 
+    (PyArray_VectorUnaryFunc*)CDOUBLE_to_UINT, 
     (PyArray_VectorUnaryFunc*)CDOUBLE_to_LONG, 
     (PyArray_VectorUnaryFunc*)CDOUBLE_to_FLOAT, 
     (PyArray_VectorUnaryFunc*)CDOUBLE_to_DOUBLE, 
@@ -585,6 +822,7 @@ static PyArray_Descr OBJECT_Descr = { {
     (PyArray_VectorUnaryFunc*)NULL, 
     (PyArray_VectorUnaryFunc*)NULL, 
     (PyArray_VectorUnaryFunc*)NULL, 
+    (PyArray_VectorUnaryFunc*)NULL, 
 },
 				      OBJECT_getitem, OBJECT_setitem,
 				      PyArray_OBJECT, sizeof(PyObject*), NULL, NULL, 'O'};
@@ -595,7 +833,9 @@ static PyArray_Descr *descrs[] = {
     &UBYTE_Descr, 
     &SBYTE_Descr, 
     &SHORT_Descr, 
+    &USHORT_Descr, 
     &INT_Descr, 
+    &UINT_Descr, 
     &LONG_Descr, 
     &FLOAT_Descr, 
     &DOUBLE_Descr, 
@@ -605,27 +845,29 @@ static PyArray_Descr *descrs[] = {
 };
 
 PyArray_Descr *PyArray_DescrFromType(int type) {
-    if (type < PyArray_NTYPES) {
-	return descrs[type];
-    } else {
-	switch(type) {
-	case 'c': return descrs[PyArray_CHAR];
-	case 'b': return descrs[PyArray_UBYTE];
-	case '1': return descrs[PyArray_SBYTE];
-	case 's': return descrs[PyArray_SHORT];
-	case 'i': return descrs[PyArray_INT];
-	case 'l': return descrs[PyArray_LONG];
-	case 'f': return descrs[PyArray_FLOAT];
-	case 'd': return descrs[PyArray_DOUBLE];
-	case 'F': return descrs[PyArray_CFLOAT];
-	case 'D': return descrs[PyArray_CDOUBLE];
-	case 'O': return descrs[PyArray_OBJECT];
-
-	default:
-	    PyErr_SetString(PyExc_ValueError, "Invalid type for array");	
-	    return NULL;
+	if (type < PyArray_NTYPES) {
+		return descrs[type];
+	} else {
+		switch(type) {
+		case 'c': return descrs[PyArray_CHAR];
+		case 'b': return descrs[PyArray_UBYTE];
+		case '1': return descrs[PyArray_SBYTE];
+		case 's': return descrs[PyArray_SHORT];
+		case 'w': return descrs[PyArray_USHORT];
+		case 'i': return descrs[PyArray_INT];
+		case 'u': return descrs[PyArray_UINT];
+		case 'l': return descrs[PyArray_LONG];
+		case 'f': return descrs[PyArray_FLOAT];
+		case 'd': return descrs[PyArray_DOUBLE];
+		case 'F': return descrs[PyArray_CFLOAT];
+		case 'D': return descrs[PyArray_CDOUBLE];
+		case 'O': return descrs[PyArray_OBJECT];
+			
+		default:
+			PyErr_SetString(PyExc_ValueError, "Invalid type for array");	
+			return NULL;
+		}
 	}
-    }
 }
 
 
