@@ -1127,6 +1127,79 @@ array_array(PyObject *ignored, PyObject *args, PyObject *kws)
     return ret;
 }
 
+static char doc_empty[] = "empty((d1,...,dn),typecode='l',savespace=0) will return a new array\n"\
+    "of shape (d1,...,dn) and given type with all its entries uninitialized.  If savespace is\n" \
+    "nonzero, the array will be a spacesaver array.  This can be faster than zeros.";
+
+static PyObject *array_empty(PyObject *ignored, PyObject *args, PyObject *kwds) {
+
+    PyObject *sequence;
+    char type='l';
+    int savespace=0;
+    static char *kwlist[] = {"shape", "typecode", "savespace", NULL};
+    PyObject *op;
+    PyArray_Descr *descr;
+    int i, nd, n, dims[MAX_DIMS];
+    int sd;
+    char *data;
+
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "O|ci", kwlist,
+                                     &sequence, &type, &savespace)) {
+	return NULL;
+    }
+    
+    if ((descr = PyArray_DescrFromType((int)type)) == NULL) 
+        return NULL;
+
+    if ((nd=PySequence_Length(sequence)) == -1) {
+	PyErr_Clear();
+	if (!(op = PyNumber_Int(sequence))) return NULL;
+	nd = 1;
+	dims[0] = PyInt_AsLong(op);
+	Py_DECREF(op);
+    } else {
+	if (nd > MAX_DIMS) {
+	    fprintf(stderr, "Maximum number of dimensions = %d\n", MAX_DIMS);
+	    PyErr_SetString(PyExc_ValueError, "Number of dimensions is too large");
+	    return NULL;
+	}
+	for(i=0; i<nd; i++) {
+	    if( (op=PySequence_GetItem(sequence,i))) {
+		dims[i]=PyInt_AsLong(op);
+		Py_DECREF(op);
+	    }
+	    if(PyErr_Occurred()) return NULL;
+	}
+    }
+    
+    sd = descr->elsize;
+    for(i=nd-1;i>=0;i--) 
+    {
+    	if (dims[i] < 0) {
+    	    PyErr_SetString(PyExc_ValueError, "negative dimensions are not allowed");
+            return NULL;
+        }
+        sd *= dims[i] ? dims[i] : 1;
+    }
+    /* Make sure we're alligned on ints. */
+    sd += sizeof(int) - sd%sizeof(int); 
+    
+    /* allocate memory */
+    if ((data = (char *)malloc(sd)) == NULL) {
+        PyErr_SetString(PyExc_MemoryError, "can't allocate memory for array");
+        return NULL;
+    }
+    
+    if ((op = PyArray_FromDimsAndDataAndDescr(nd, dims, descr, data))==NULL)
+        return NULL;
+    
+    /* we own the data (i.e. this informs the delete function to free the memory) */
+    ((PyArrayObject *)op)->flags |= OWN_DATA;
+    if (savespace) ((PyArrayObject *)op)->flags |= SAVESPACE;
+
+    return op;
+}
+
 static char doc_zeros[] = "zeros((d1,...,dn),typecode='l',savespace=0) will return a new array of shape (d1,...,dn) and type typecode with all it's entries initialized to zero.  If savespace is nonzero the array will be a spacesaver array.";
 
 static PyObject *array_zeros(PyObject *ignored, PyObject *args, PyObject *kwds) {
@@ -1540,6 +1613,7 @@ static struct PyMethodDef array_module_methods[] = {
     {"array",	(PyCFunction)array_array, METH_VARARGS|METH_KEYWORDS, doc_array},
     {"arange",  (PyCFunction)array_arange, METH_VARARGS|METH_KEYWORDS, doc_arange},
     {"zeros",	(PyCFunction)array_zeros, METH_VARARGS|METH_KEYWORDS, doc_zeros},
+    {"empty",	(PyCFunction)array_empty, METH_VARARGS|METH_KEYWORDS, doc_empty},
     {"fromstring",(PyCFunction)array_fromString, METH_VARARGS|METH_KEYWORDS, doc_fromString},
     {"take",	(PyCFunction)array_take, METH_VARARGS|METH_KEYWORDS, doc_take},
     {"put",	(PyCFunction)array_put, METH_VARARGS, doc_put},
