@@ -198,7 +198,7 @@ extern void PyUFunc_O_O_method(char **args, int *dimensions, int *steps, void *f
 #ifndef min
 #define min(x,y) (x)>(y)?(y):(x)
 #endif
-#define MAX_DIMS 20
+#define MAX_DIMS 30
 #define MAX_ARGS 10
 
 static int compare_lists(int *l1, int *l2, int n) {
@@ -475,7 +475,7 @@ int PyUFunc_GenericFunction(PyUFuncObject *self, PyObject *args, PyArrayObject *
     return 0;
 }
 
-PyObject * PyUFunc_GenericReduction(PyUFuncObject *self, PyObject *args, int accumulate) {
+PyObject * PyUFunc_GenericReduction(PyUFuncObject *self, PyObject *args, PyObject *kwds, int accumulate) {
     int steps[MAX_DIMS][MAX_ARGS];
     int i, j, loop, n_loops, loop_i[MAX_DIMS], loop_n[MAX_DIMS];
     char *pointers[MAX_ARGS], *resets[MAX_DIMS][MAX_ARGS], *dptr, *d;
@@ -488,6 +488,7 @@ PyObject * PyUFunc_GenericReduction(PyUFuncObject *self, PyObject *args, int acc
     long zero=0;
     int one = 1;
     PyArrayObject *indices;
+    static char *kwlist[] = {"array", "axis", NULL};
 	
 	
     if (self == NULL) {
@@ -496,7 +497,8 @@ PyObject * PyUFunc_GenericReduction(PyUFuncObject *self, PyObject *args, int acc
     }
 	
     dimension = 0;
-    if(!PyArg_ParseTuple(args, "O|i", &op, &dimension)) return NULL;
+    if(!PyArg_ParseTupleAndKeywords(args, kwds, "O|i", kwlist, 
+                                    &op, &dimension)) return NULL;
 	
 	
     /* Determine the types of the input arguments. */
@@ -890,9 +892,7 @@ PyObject *ufunc_outer(PyUFuncObject *self, PyObject *args) {
 }
 
 
-
-
-PyObject *ufunc_reduce(PyUFuncObject *self, PyObject *args) {
+PyObject *ufunc_reduce(PyUFuncObject *self, PyObject *args, PyObject *kwds) {
     if (self->nin != 2) {
 	PyErr_SetString(PyExc_ValueError, 
 			"reduce only supported for binary functions");
@@ -904,10 +904,10 @@ PyObject *ufunc_reduce(PyUFuncObject *self, PyObject *args) {
 	return NULL;
     }
 	
-    return PyUFunc_GenericReduction(self, args, 0);
+    return PyUFunc_GenericReduction(self, args, kwds, 0);
 }
 
-PyObject *ufunc_accumulate(PyUFuncObject *self, PyObject *args) {
+PyObject *ufunc_accumulate(PyUFuncObject *self, PyObject *args, PyObject *kwds) {
     if (self->nin != 2) {
 	PyErr_SetString(PyExc_ValueError, 
 			"accumulate only supported for binary functions");
@@ -919,7 +919,7 @@ PyObject *ufunc_accumulate(PyUFuncObject *self, PyObject *args) {
 	return NULL;
     }
 	
-    return PyUFunc_GenericReduction(self, args, 1);
+    return PyUFunc_GenericReduction(self, args, kwds, 1);
 }
 
 PyObject *ufunc_reduceAt(PyUFuncObject *self, PyObject *args) {
@@ -939,11 +939,11 @@ PyObject *ufunc_reduceAt(PyUFuncObject *self, PyObject *args) {
 
 
 static struct PyMethodDef ufunc_methods[] = {
-    {"reduce",  (PyCFunction)ufunc_reduce, 1},
-    {"accumulate",  (PyCFunction)ufunc_accumulate, 1},
-    {"reduceat",  (PyCFunction)ufunc_reduceAt, 1},
+    {"reduce",  (PyCFunction)ufunc_reduce, METH_VARARGS | METH_KEYWORDS},
+    {"accumulate",  (PyCFunction)ufunc_accumulate, METH_VARARGS | METH_KEYWORDS},
+    {"reduceat",  (PyCFunction)ufunc_reduceAt, METH_VARARGS},
 	
-    {"outer", (PyCFunction)ufunc_outer, 1},
+    {"outer", (PyCFunction)ufunc_outer, METH_VARARGS},
     {NULL,		NULL}		/* sentinel */
 };
 
